@@ -28,6 +28,7 @@ from bcbio.distributed import multi, prun
 from bcbio.distributed.split import parallel_split_combine
 from bcbio.distributed.transaction import file_transaction
 from bcbio.pipeline import shared
+from bcbio.variation import multi as vmulti
 
 def parallel_callable_loci(in_bam, ref_file, config):
     num_cores = config["algorithm"].get("num_cores", 1)
@@ -36,7 +37,7 @@ def parallel_callable_loci(in_bam, ref_file, config):
     data = {"work_bam": in_bam, "sam_ref": ref_file, "config": config}
     parallel = {"type": "local", "cores": num_cores, "module": "bcbio.distributed"}
     items = [[data]]
-    with prun.start(parallel, items, config) as runner:
+    with prun.start(parallel, items, config, multiplier=int(num_cores)) as runner:
         split_fn = shared.process_bam_by_chromosome("-callable.bed", "work_bam")
         out = parallel_split_combine(items, split_fn, runner,
                                      "calc_callable_loci", "combine_bed",
@@ -68,10 +69,15 @@ def calc_callable_loci(data, region=None, out_file=None):
 
 def _get_coverage_in_region(in_bam, region, depth):
     """Retrieve summary of coverage in a region.
+    Requires positive non-zero mapping quality at a position, matching GATK's
+    CallableLoci defaults.
+
     Uses chajo's approach of pre-allocating a numpy array for coverage before collapsing
     to cleanly handle non-covered positions. This uses 2Gb memory for human chr1. If memory
     requirements become an issue, could look at splitting long chromosomes at smart places.
-    XXX Can replace `positions` with chanjo functionality when it accepts max_depth keyword.
+
+    XXX Can replace `positions` with chanjo functionality when it accepts max_depth keyword
+    and handles mapping quality.
     """
     # special case, do not calculate if we are in a chromosome not covered by BED file
     if region.attrs.get("no_coverage"):
@@ -80,7 +86,8 @@ def _get_coverage_in_region(in_bam, region, depth):
         positions = numpy.zeros(region.end + 1 - region.start)
         for col in in_bam.pileup(str(region.chrom), region.start, region.end + 1, stepper="all",
                                  max_depth=depth["max"] + 10, truncate=True):
-            positions[col.pos - region.start] = col.n
+            n = sum(1 for read in col.pileups if read.alignment.mapq > 0)
+            positions[col.pos - region.start] = n
         cur_ctype = None
         cur_start = None
         for i, count in enumerate(positions):
@@ -145,7 +152,7 @@ def sample_callable_bed(bam_file, ref_file, config):
                 filter_regions.saveas(tx_out_file)
     return out_file
 
-def get_ref_bedtool(ref_file, config):
+def get_ref_bedtool(ref_file, config, chrom=None):
     """Retrieve a pybedtool BedTool object with reference sizes from input reference.
     """
     broad_runner = broad.runner_from_config(config)
@@ -153,7 +160,8 @@ def get_ref_bedtool(ref_file, config):
     ref_lines = []
     with contextlib.closing(pysam.Samfile(ref_dict, "r")) as ref_sam:
         for sq in ref_sam.header["SQ"]:
-            ref_lines.append("%s\t%s\t%s" % (sq["SN"], 0, sq["LN"]))
+            if not chrom or sq["SN"] == chrom:
+                ref_lines.append("%s\t%s\t%s" % (sq["SN"], 0, sq["LN"]))
     return pybedtools.BedTool("\n".join(ref_lines), from_string=True)
 
 def _get_nblock_regions(in_file, min_n_size):
@@ -222,6 +230,7 @@ class NBlockRegionPicker:
         self._chr_last_blocks = {}
         target_blocks = int(config["algorithm"].get("nomap_split_targets", 2000))
         self._target_size = self._get_target_size(target_blocks, ref_regions)
+        self._ref_sizes = {x.chrom: x.stop for x in ref_regions}
 
     def _get_target_size(self, target_blocks, ref_regions):
         size = 0
@@ -236,6 +245,9 @@ class NBlockRegionPicker:
         if (x.start - last_pos) > self._target_size:
             self._chr_last_blocks[x.chrom] = x.stop
             return True
+        # fills an entire chromosome, handles smaller decoy and haplotype chromosomes
+        elif last_pos == 0 and x.stop >= self._ref_sizes.get(x.chrom, 0) - 100:
+            return True
         else:
             return False
 
@@ -254,7 +266,11 @@ def block_regions(in_bam, ref_file, config):
         nblock_regions = _get_nblock_regions(callable_bed, min_n_size)
         nblock_regions = _add_config_regions(nblock_regions, ref_regions, config)
         nblock_regions.saveas(nblock_bed)
-        ref_regions.subtract(nblock_bed).merge(d=min_n_size).saveas(callblock_bed)
+        if len(ref_regions.subtract(nblock_regions)) > 0:
+            ref_regions.subtract(nblock_bed).merge(d=min_n_size).saveas(callblock_bed)
+        else:
+            raise ValueError("No callable regions found from BAM file. Alignment regions might "
+                             "not overlap with regions found in your `variant_regions` BED: %s" % in_bam)
     return callblock_bed, nblock_bed, callable_bed
 
 def _write_bed_regions(sample, final_regions, out_file, out_file_ref):
@@ -322,41 +338,64 @@ def _combine_excessive_coverage(samples, ref_regions, min_n_size):
         return merge_ecs
 
 def combine_sample_regions(*samples):
-    """Create global set of callable regions for multi-sample calling.
+    """Create batch-level sets of callable regions for multi-sample calling.
 
-    Intersects all non-callable (nblock) regions from all samples,
+    Intersects all non-callable (nblock) regions from all samples in a batch,
     producing a global set of callable regions.
     """
-    config = samples[0]["config"]
-    work_dir = samples[0]["dirs"]["work"]
-    analysis_file = os.path.join(work_dir, "analysis_blocks.bed")
-    no_analysis_file = os.path.join(work_dir, "noanalysis_blocks.bed")
-    min_n_size = int(config["algorithm"].get("nomap_split_size", 100))
+    # back compatibility -- global file for entire sample set
+    global_analysis_file = os.path.join(samples[0]["dirs"]["work"], "analysis_blocks.bed")
+    if utils.file_exists(global_analysis_file) and not _needs_region_update(global_analysis_file, samples):
+        global_no_analysis_file = os.path.join(os.path.dirname(global_analysis_file), "noanalysis_blocks.bed")
+    else:
+        global_analysis_file = None
+    out = []
+    analysis_files = []
+    for batch, items in vmulti.group_by_batch(samples).items():
+        if global_analysis_file:
+            analysis_file, no_analysis_file = global_analysis_file, global_no_analysis_file
+        else:
+            analysis_file, no_analysis_file = _combine_sample_regions_batch(batch, items)
+        for data in items:
+            if analysis_file:
+                analysis_files.append(analysis_file)
+                data["config"]["algorithm"]["callable_regions"] = analysis_file
+                data["config"]["algorithm"]["non_callable_regions"] = no_analysis_file
+            out.append([data])
+    assert len(out) == len(samples)
+    final_regions = pybedtools.BedTool(analysis_files[0])
+    _analysis_block_stats(final_regions)
+    return out
 
-    if not utils.file_exists(analysis_file) or _needs_region_update(analysis_file, samples):
+def _combine_sample_regions_batch(batch, items):
+    """Combine sample regions within a group of batched samples.
+    """
+    config = items[0]["config"]
+    work_dir = utils.safe_makedir(os.path.join(items[0]["dirs"]["work"], "regions"))
+    analysis_file = os.path.join(work_dir, "%s-analysis_blocks.bed" % batch)
+    no_analysis_file = os.path.join(work_dir, "%s-noanalysis_blocks.bed" % batch)
+    if not utils.file_exists(analysis_file) or _needs_region_update(analysis_file, items):
         # Combine all nblocks into a final set of intersecting regions
         # without callable bases. HT @brentp for intersection approach
         # https://groups.google.com/forum/?fromgroups#!topic/bedtools-discuss/qA9wK4zN8do
-        nblock_regions = reduce(operator.add,
-                                (pybedtools.BedTool(x["regions"]["nblock"])
-                                 for x in samples if "regions" in x))
-        ref_regions = get_ref_bedtool(samples[0]["sam_ref"], config)
-        ec_regions = _combine_excessive_coverage(samples, ref_regions, min_n_size)
-        block_filter = NBlockRegionPicker(ref_regions, config)
-        nblock_size_filtered = nblock_regions.filter(block_filter.include_block).saveas()
-        if len(nblock_size_filtered) > len(ref_regions):
-            final_nblock_regions = nblock_size_filtered
+        bed_regions = [pybedtools.BedTool(x["regions"]["nblock"])
+                       for x in items if "regions" in x]
+        if len(bed_regions) == 0:
+            analysis_file, no_analysis_file = None, None
         else:
-            final_nblock_regions = nblock_regions
-        final_regions = ref_regions.subtract(final_nblock_regions)
-        if len(ec_regions) > 0:
-            final_regions = final_regions.subtract(ec_regions)
-        final_regions.merge(d=min_n_size)
-        _write_bed_regions(samples[0], final_regions, analysis_file, no_analysis_file)
-    else:
-        final_regions = pybedtools.BedTool(analysis_file)
-    _analysis_block_stats(final_regions)
-    regions = {"analysis": [(r.chrom, int(r.start), int(r.stop)) for r in final_regions],
-               "noanalysis": no_analysis_file,
-               "analysis_bed": analysis_file}
-    return [regions]
+            nblock_regions = reduce(operator.add, bed_regions)
+            ref_regions = get_ref_bedtool(items[0]["sam_ref"], config)
+            min_n_size = int(config["algorithm"].get("nomap_split_size", 100))
+            ec_regions = _combine_excessive_coverage(items, ref_regions, min_n_size)
+            block_filter = NBlockRegionPicker(ref_regions, config)
+            nblock_size_filtered = nblock_regions.filter(block_filter.include_block).saveas()
+            if len(nblock_size_filtered) >= len(ref_regions):
+                final_nblock_regions = nblock_size_filtered
+            else:
+                final_nblock_regions = nblock_regions
+            final_regions = ref_regions.subtract(final_nblock_regions)
+            if len(ec_regions) > 0:
+                final_regions = final_regions.subtract(ec_regions)
+            final_regions.merge(d=min_n_size)
+            _write_bed_regions(items[0], final_regions, analysis_file, no_analysis_file)
+    return analysis_file, no_analysis_file
