@@ -6,12 +6,12 @@ from IPython.parallel import require
 
 from bcbio.bam import callable
 from bcbio.ngsalign import alignprep
-from bcbio.pipeline import (config_utils, disambiguate, sample, lane, qcsummary, shared,
+from bcbio.pipeline import (archive, config_utils, disambiguate, sample, lane, qcsummary, shared,
                             variation, rnaseq)
 from bcbio.provenance import system
 from bcbio import structural
 from bcbio import chipseq
-from bcbio.variation import (bamprep, coverage, realign, genotype, ensemble, multi, population,
+from bcbio.variation import (bamprep, coverage, genotype, ensemble, multi, population,
                              recalibrate, validate, vcfutils)
 from bcbio.log import logger, setup_local_logging
 
@@ -27,8 +27,11 @@ def _setup_logging(args):
         elif config_utils.is_std_config_arg(arg):
             config = arg
             break
+        elif isinstance(arg, (list, tuple)) and config_utils.is_nested_config_arg(arg[0]):
+            config = arg[0]["config"]
+            break
     if config is None:
-        raise NotImplementedError("No config in %s:" % args[0])
+        raise NotImplementedError("No config found in arguments: %s" % args[0])
     handler = setup_local_logging(config, config.get("parallel", {}))
     try:
         yield None
@@ -84,16 +87,6 @@ def prep_recal(*args):
     with _setup_logging(args):
         return apply(recalibrate.prep_recal, *args)
 
-@require(recalibrate)
-def write_recal_bam(*args):
-    with _setup_logging(args):
-        return apply(recalibrate.write_recal_bam, *args)
-
-@require(realign)
-def realign_sample(*args):
-    with _setup_logging(args):
-        return apply(realign.realign_sample, *args)
-
 @require(multi)
 def split_variants_by_sample(*args):
     with _setup_logging(args):
@@ -124,11 +117,6 @@ def run_cufflinks(*args):
     with _setup_logging(args):
         return apply(rnaseq.run_cufflinks, *args)
 
-@require(sample)
-def generate_bigwig(*args):
-    with _setup_logging(args):
-        return apply(sample.generate_bigwig, *args)
-
 @require(shared)
 def combine_bam(*args):
     with _setup_logging(args):
@@ -196,3 +184,8 @@ def machine_info(*args):
 @require(chipseq)
 def clean_chipseq_alignment(*args):
     return chipseq.machine_info()
+
+@require(archive)
+def archive_to_cram(*args):
+    with _setup_logging(args):
+        return apply(archive.to_cram, *args)
