@@ -46,27 +46,32 @@ def upgrade_bcbio(args):
         print("Upgrading bcbio-nextgen to latest stable version")
         sudo_cmd = [] if args.upgrade == "stable" else ["sudo"]
         subprocess.check_call(sudo_cmd + [pip_bin, "install", "-r", REMOTES["requirements"]])
+        print("Upgrade of bcbio-nextgen code complete.")
     else:
         _update_conda_packages()
         print("Upgrading bcbio-nextgen to latest development version")
         subprocess.check_call([pip_bin, "install", "git+%s#egg=bcbio-nextgen" % REMOTES["gitrepo"]])
         subprocess.check_call([pip_bin, "install", "--upgrade", "--no-deps",
                                "git+%s#egg=bcbio-nextgen" % REMOTES["gitrepo"]])
+        print("Upgrade of bcbio-nextgen development code complete.")
 
     if args.tooldir:
         with bcbio_tmpdir():
             print("Upgrading third party tools to latest versions")
             upgrade_thirdparty_tools(args, REMOTES)
+            print("Third party tools upgrade complete.")
     if args.install_data:
         with bcbio_tmpdir():
             print("Upgrading bcbio-nextgen data files")
             upgrade_bcbio_data(args, REMOTES)
+            print("bcbio-nextgen data upgrade complete.")
     if args.isolate and args.tooldir:
         print("Installation directory not added to current PATH")
         print("  Add {t}/bin to PATH and {t}/lib to LD_LIBRARY_PATH".format(t=args.tooldir))
     save_install_defaults(args)
     args.datadir = _get_data_dir()
     _install_container_bcbio_system(args.datadir)
+    print("Upgrade completed successfully.")
     return args
 
 def _install_container_bcbio_system(datadir):
@@ -122,7 +127,7 @@ def _update_conda_packages():
     pkgs = ["biopython", "boto", "cython", "ipython", "lxml", "matplotlib",
             "nose", "numpy", "pandas", "patsy", "pycrypto", "pip", "pysam",
             "pyyaml", "pyzmq", "requests", "scipy", "setuptools", "sqlalchemy",
-            "statsmodels", "tornado"]
+            "statsmodels", "toolz", "tornado"]
     channels = ["-c", "https://conda.binstar.org/collections/chapmanb/bcbio"]
     if os.path.exists(conda_bin):
         subprocess.check_call([conda_bin, "install", "--yes", "numpy"])
@@ -232,19 +237,20 @@ def upgrade_thirdparty_tools(args, remotes):
     s["actions"] = ["install_biolinux"]
     s["fabricrc_overrides"]["system_install"] = args.tooldir
     s["fabricrc_overrides"]["local_install"] = os.path.join(args.tooldir, "local_install")
-    manifest_dir = os.path.join(_get_data_dir(), "manifest")
-    _install_toolplus(args, manifest_dir)
     cbl = get_cloudbiolinux(remotes)
     sys.path.insert(0, cbl["dir"])
     cbl_deploy = __import__("cloudbio.deploy", fromlist=["deploy"])
     cbl_deploy.deploy(s)
-    cbl_manifest = __import__("cloudbio.manifest", fromlist=["manifest"])
+    manifest_dir = os.path.join(_get_data_dir(), "manifest")
     print("Creating manifest of installed packages in %s" % manifest_dir)
+    cbl_manifest = __import__("cloudbio.manifest", fromlist=["manifest"])
     if os.path.exists(manifest_dir):
         for fname in os.listdir(manifest_dir):
             if not fname.startswith("toolplus"):
                 os.remove(os.path.join(manifest_dir, fname))
     cbl_manifest.create(manifest_dir, args.tooldir)
+    print("Installing additional tools")
+    _install_toolplus(args, manifest_dir)
 
 def _install_toolplus(args, manifest_dir):
     """Install additional tools we cannot distribute, updating local manifest.
@@ -442,7 +448,7 @@ def add_subparser(subparsers):
                         action="append", default=[], type=_check_toolplus)
     parser.add_argument("--genomes", help="Genomes to download",
                         action="append", default=["GRCh37"],
-                        choices=["GRCh37", "hg19", "mm10", "mm9", "rn5", "canFam3"])
+                        choices=["GRCh37", "hg19", "mm10", "mm9", "rn5", "canFam3", "dm3", "Zv9", "phix"])
     parser.add_argument("--aligners", help="Aligner indexes to download",
                         action="append", default=["bwa", "bowtie2"],
                         choices=["bowtie", "bowtie2", "bwa", "novoalign", "star", "ucsc"])
