@@ -13,16 +13,15 @@ import tempfile
 
 from bcbio import install, log, structural, utils, upload
 from bcbio.distributed import clargs, prun, runfn
+from bcbio.illumina import flowcell, machine
 from bcbio.log import logger
 from bcbio.ngsalign import alignprep
-from bcbio.pipeline import (disambiguate, region, run_info, qcsummary,
+from bcbio.pipeline import (archive, disambiguate, region, run_info, qcsummary,
                             version, rnaseq)
 from bcbio.pipeline.config_utils import load_system_config
 from bcbio.provenance import programs, profile, system, versioncheck
 from bcbio.server import main as server_main
-from bcbio.solexa.flowcell import get_fastq_dir
-from bcbio.variation.genotype import combine_multiple_callers
-from bcbio.variation import coverage, ensemble, population, validate
+from bcbio.variation import coverage, ensemble, genotype, population, validate
 from bcbio.rnaseq.count import (combine_count_files,
                                 annotate_combined_count_file)
 
@@ -34,7 +33,7 @@ def run_main(workdir, config_file=None, fc_dir=None, run_info_yaml=None,
     config, config_file = load_system_config(config_file, workdir)
     if config.get("log_dir", None) is None:
         config["log_dir"] = os.path.join(workdir, "log")
-    if parallel["type"] in ["local"]:
+    if parallel["type"] in ["local", "clusterk"]:
         _setup_resources()
         _run_toplevel(config, config_file, workdir, parallel,
                       fc_dir, run_info_yaml)
@@ -52,12 +51,12 @@ def _setup_resources():
     This allows us to avoid out of file handle limits where we can
     move beyond the soft limit up to the hard limit.
     """
-    target_procs = 50000
+    target_procs = 10240
     cur_proc, max_proc = resource.getrlimit(resource.RLIMIT_NPROC)
-    target_proc = min(max_proc, target_procs)
+    target_proc = min(max_proc, target_procs) if max_proc > 0 else target_procs
     resource.setrlimit(resource.RLIMIT_NPROC, (max(cur_proc, target_proc), max_proc))
     cur_hdls, max_hdls = resource.getrlimit(resource.RLIMIT_NOFILE)
-    target_hdls = min(max_hdls, target_procs)
+    target_hdls = min(max_hdls, target_procs) if max_hdls > 0 else target_procs
     resource.setrlimit(resource.RLIMIT_NOFILE, (max(cur_hdls, target_hdls), max_hdls))
 
 def _run_toplevel(config, config_file, work_dir, parallel,
@@ -70,16 +69,12 @@ def _run_toplevel(config, config_file, work_dir, parallel,
     """
     parallel = log.create_base_logger(config, parallel)
     log.setup_local_logging(config, parallel)
-    fastq_dir, galaxy_dir, config_dir = _get_full_paths(get_fastq_dir(fc_dir)
-                                                        if fc_dir else None,
-                                                        config, config_file)
-    config_file = os.path.join(config_dir, os.path.basename(config_file))
-    dirs = {"fastq": fastq_dir, "galaxy": galaxy_dir,
-            "work": work_dir, "flowcell": fc_dir, "config": config_dir}
+    dirs = setup_directories(work_dir, fc_dir, config, config_file)
+    config_file = os.path.join(dirs["config"], os.path.basename(config_file))
     samples = run_info.organize(dirs, config, run_info_yaml)
     pipelines = _pair_lanes_with_pipelines(samples)
     final = []
-    with utils.curdir_tmpdir() as tmpdir:
+    with utils.curdir_tmpdir({"config": config}) as tmpdir:
         tempfile.tempdir = tmpdir
         for pipeline, pipeline_items in pipelines.items():
             pipeline_items = _add_provenance(pipeline_items, dirs, parallel, config)
@@ -89,6 +84,13 @@ def _run_toplevel(config, config_file, work_dir, parallel,
                     upload.from_sample(xs[0])
                     final.append(xs[0])
 
+def setup_directories(work_dir, fc_dir, config, config_file):
+    fastq_dir, galaxy_dir, config_dir = _get_full_paths(flowcell.get_fastq_dir(fc_dir)
+                                                        if fc_dir else None,
+                                                        config, config_file)
+    return {"fastq": fastq_dir, "galaxy": galaxy_dir,
+            "work": work_dir, "flowcell": fc_dir, "config": config_dir}
+
 def _add_provenance(items, dirs, parallel, config):
     p = programs.write_versions(dirs, config, is_wrapper=parallel.get("wrapper") is not None)
     system.write_info(dirs, parallel, config)
@@ -118,6 +120,13 @@ def _sanity_check_args(args):
         elif args.paralleltype == "ipython" and (not args.queue or not args.scheduler):
             return "IPython parallel requires queue (-q) and scheduler (-s) arguments."
 
+def _sanity_check_kwargs(args):
+    """Sanity check after setting up input arguments, handling back compatibility
+    """
+    if not args.get("workflow") and not args.get("run_info_yaml"):
+        return ("Require a sample YAML file describing inputs: "
+                "https://bcbio-nextgen.readthedocs.org/en/latest/contents/configuration.html")
+
 def parse_cl_args(in_args):
     """Parse input commandline arguments, handling multiple cases.
 
@@ -126,7 +135,8 @@ def parse_cl_args(in_args):
     sub_cmds = {"upgrade": install.add_subparser,
                 "server": server_main.add_subparser,
                 "runfn": runfn.add_subparser,
-                "version": programs.add_subparser}
+                "version": programs.add_subparser,
+                "sequencer": machine.add_subparser}
     parser = argparse.ArgumentParser(
         description="Best-practice pipelines for fully automated high throughput sequencing analysis.")
     sub_cmd = None
@@ -176,6 +186,9 @@ def parse_cl_args(in_args):
                   "workflow": args.workflow,
                   "workdir": args.workdir}
         kwargs = _add_inputs_to_kwargs(args, kwargs, parser)
+        error_msg = _sanity_check_kwargs(kwargs)
+        if error_msg:
+            parser.error(error_msg)
     else:
         assert sub_cmd is not None
         kwargs = {"args": args,
@@ -293,7 +306,7 @@ class Variant2Pipeline(AbstractPipeline):
     @classmethod
     def run(self, config, config_file, parallel, dirs, samples):
         ## Alignment and preparation requiring the entire input file (multicore cluster)
-        with prun.start(_wres(parallel, ["aligner", "gatk"],
+        with prun.start(_wres(parallel, ["aligner", "samtools", "sambamba"],
                               (["reference", "fasta"], ["reference", "aligner"], ["files"])),
                         samples, config, dirs, "multicore",
                         multiplier=alignprep.parallel_multiplier(samples)) as run_parallel:
@@ -306,8 +319,7 @@ class Variant2Pipeline(AbstractPipeline):
                 samples = disambiguate.resolve(samples, run_parallel)
             with profile.report("callable regions", dirs):
                 samples = run_parallel("postprocess_alignment", samples)
-                regions = run_parallel("combine_sample_regions", [samples])[0]
-                samples = region.add_region_info(samples, regions)
+                samples = run_parallel("combine_sample_regions", [samples])
                 samples = region.clean_sample_data(samples)
             with profile.report("coverage", dirs):
                 samples = coverage.summarize_samples(samples, run_parallel)
@@ -315,20 +327,21 @@ class Variant2Pipeline(AbstractPipeline):
         ## Variant calling on sub-regions of the input file (full cluster)
         with prun.start(_wres(parallel, ["gatk", "picard", "variantcaller"]),
                         samples, config, dirs, "full",
-                        multiplier=len(regions["analysis"]), max_multicore=1) as run_parallel:
+                        multiplier=region.get_max_counts(samples), max_multicore=1) as run_parallel:
             with profile.report("alignment post-processing", dirs):
-                samples = region.parallel_prep_region(samples, regions, run_parallel)
+                samples = region.parallel_prep_region(samples, run_parallel)
             with profile.report("variant calling", dirs):
-                samples = region.parallel_variantcall_region(samples, run_parallel)
+                samples = genotype.parallel_variantcall_region(samples, run_parallel)
 
         ## Finalize variants (per-sample cluster)
         with prun.start(_wres(parallel, ["gatk", "gatk-vqsr", "snpeff", "bcbio_variation"]),
                         samples, config, dirs, "persample") as run_parallel:
             with profile.report("variant post-processing", dirs):
                 samples = run_parallel("postprocess_variants", samples)
+                samples = run_parallel("split_variants_by_sample", samples)
             with profile.report("validation", dirs):
                 samples = run_parallel("compare_to_rm", samples)
-                samples = combine_multiple_callers(samples)
+                samples = genotype.combine_multiple_callers(samples)
         ## Finalizing BAMs and population databases, handle multicore computation
         with prun.start(_wres(parallel, ["gemini", "samtools", "fastqc", "bamtools", "bcbio_variation",
                                          "bcbio-variation-recall"]),
@@ -337,6 +350,7 @@ class Variant2Pipeline(AbstractPipeline):
                 samples = region.delayed_bamprep_merge(samples, run_parallel)
             with profile.report("ensemble calling", dirs):
                 samples = ensemble.combine_calls_parallel(samples, run_parallel)
+            with profile.report("validation summary", dirs):
                 samples = validate.summarize_grading(samples)
             with profile.report("structural variation", dirs):
                 samples = structural.run(samples, run_parallel)
@@ -344,9 +358,19 @@ class Variant2Pipeline(AbstractPipeline):
                 samples = population.prep_db_parallel(samples, run_parallel)
             with profile.report("quality control", dirs):
                 samples = qcsummary.generate_parallel(samples, run_parallel)
+            with profile.report("archive", dirs):
+                samples = archive.compress(samples, run_parallel)
         logger.info("Timing: finished")
         return samples
 
+def _debug_samples(i, samples):
+    print "---", i, len(samples)
+    for sample in (x[0] for x in samples):
+        print "  ", sample["description"], sample.get("region"), \
+            utils.get_in(sample, ("config", "algorithm", "variantcaller")), \
+            [x.get("variantcaller") for x in sample.get("variants", [])], \
+            sample.get("work_bam")
+
 class SNPCallingPipeline(Variant2Pipeline):
     """Back compatible: old name for variant analysis.
     """
@@ -366,13 +390,17 @@ class StandardPipeline(AbstractPipeline):
         ## Alignment and preparation requiring the entire input file (multicore cluster)
         with prun.start(_wres(parallel, ["aligner"]),
                         lane_items, config, dirs, "multicore") as run_parallel:
-            logger.info("Timing: alignment")
-            samples = run_parallel("process_alignment", lane_items)
-        ## Finalize (per-sample cluster)
-        with prun.start(_wres(parallel, ["fastqc", "bamtools"]),
-                        samples, config, dirs, "persample") as run_parallel:
-            logger.info("Timing: quality control")
-            samples = qcsummary.generate_parallel(samples, run_parallel)
+            with profile.report("alignment", dirs):
+                samples = run_parallel("process_alignment", lane_items)
+            with profile.report("callable regions", dirs):
+                samples = run_parallel("postprocess_alignment", samples)
+                samples = run_parallel("combine_sample_regions", [samples])
+                samples = region.clean_sample_data(samples)
+        ## Quality control
+        with prun.start(_wres(parallel, ["fastqc", "bamtools", "samtools"]),
+                        samples, config, dirs, "multicore2") as run_parallel:
+            with profile.report("quality control", dirs):
+                samples = qcsummary.generate_parallel(samples, run_parallel)
         logger.info("Timing: finished")
         return samples
 
@@ -417,6 +445,7 @@ class RnaseqPipeline(AbstractPipeline):
                         samples, config, dirs, "persample") as run_parallel:
             with profile.report("quality control", dirs):
                 samples = qcsummary.generate_parallel(samples, run_parallel)
+        logger.info("Timing: finished")
         return samples
 
 class ChipseqPipeline(AbstractPipeline):
