@@ -1,5 +1,7 @@
 """Quality control and summary metrics for next-gen alignments and analysis.
 """
+import collections
+import csv
 import os
 import shutil
 import subprocess
@@ -19,13 +21,13 @@ except ImportError:
 from bcbio import bam, utils
 from bcbio.distributed.transaction import file_transaction
 from bcbio.log import logger
-from bcbio.pipeline import config_utils
+from bcbio.pipeline import config_utils, run_info
 from bcbio.provenance import do
 import bcbio.rnaseq.qc
 from bcbio.variation.realign import has_aligned_reads
 from bcbio.rnaseq.coverage import plot_gene_coverage
 
-# ## High level functions to generate summary PDF
+# ## High level functions to generate summary
 
 def generate_parallel(samples, run_parallel):
     """Provide parallel preparation of summary information for alignment and variant calling.
@@ -38,19 +40,42 @@ def generate_parallel(samples, run_parallel):
             data[0]["summary"] = {}
         data[0]["summary"]["project"] = summary_file
         samples.append(data)
+    samples = _add_researcher_summary(samples, summary_file)
     return samples
 
 def pipeline_summary(data):
     """Provide summary information on processing sample.
     """
-    work_bam = (data.get("work_bam")
-                if data["config"]["algorithm"].get("merge_bamprep", True)
-                else data.get("callable_bam"))
+    work_bam = data.get("work_bam")
     if data["sam_ref"] is not None and work_bam and has_aligned_reads(work_bam):
         logger.info("Generating summary files: %s" % str(data["name"]))
         data["summary"] = _run_qc_tools(work_bam, data)
     return [[data]]
 
+def prep_pdf(qc_dir, config):
+    """Create PDF from HTML summary outputs in QC directory.
+
+    Requires wkhtmltopdf installed: http://www.msweet.org/projects.php?Z1
+    Thanks to: https://www.biostars.org/p/16991/
+
+    Works around issues with CSS conversion on CentOS by adjusting CSS.
+    """
+    html_file = os.path.join(qc_dir, "fastqc", "fastqc_report.html")
+    html_fixed = "%s-fixed%s" % os.path.splitext(html_file)
+    try:
+        topdf = config_utils.get_program("wkhtmltopdf", config)
+    except config_utils.CmdNotFound:
+        topdf = None
+    if topdf and utils.file_exists(html_file):
+        out_file = "%s.pdf" % os.path.splitext(html_file)[0]
+        if not utils.file_exists(out_file):
+            cmd = ("sed 's/div.summary/div.summary-no/' %s | sed 's/div.main/div.main-no/' > %s"
+                   % (html_file, html_fixed))
+            do.run(cmd, "Fix fastqc CSS to be compatible with wkhtmltopdf")
+            cmd = [topdf, html_fixed, out_file]
+            do.run(cmd, "Convert QC HTML to PDF")
+        return out_file
+
 def _run_qc_tools(bam_file, data):
     """Run a set of third party quality control tools, returning QC directory and metrics.
     """
@@ -63,7 +88,7 @@ def _run_qc_tools(bam_file, data):
         to_run.append(["bamtools", _run_bamtools_stats])
     else:
         to_run += [("bamtools", _run_bamtools_stats), ("gemini", _run_gemini_stats)]
-    qc_dir = utils.safe_makedir(os.path.join(data["dirs"]["work"], "qc", data["name"][-1]))
+    qc_dir = utils.safe_makedir(os.path.join(data["dirs"]["work"], "qc", data["description"]))
     metrics = {}
     for program_name, qc_fn in to_run:
         cur_qc_dir = os.path.join(qc_dir, program_name)
@@ -91,17 +116,17 @@ def write_project_summary(samples):
     date = str(datetime.now())
     prev_samples = _other_pipeline_samples(out_file, samples)
     with open(out_file, "w") as out_handle:
-        yaml.dump({"date": date}, out_handle,
-                  default_flow_style=False, allow_unicode=False)
+        yaml.safe_dump({"date": date}, out_handle,
+                       default_flow_style=False, allow_unicode=False)
         if test_run:
-            yaml.dump({"test_run": True}, out_handle, default_flow_style=False,
-                      allow_unicode=False)
-        yaml.dump({"upload": upload_dir}, out_handle,
-                  default_flow_style=False, allow_unicode=False)
-        yaml.dump({"bcbio_system": samples[0][0]["config"].get("bcbio_system", "")}, out_handle,
-                  default_flow_style=False, allow_unicode=False)
-        yaml.dump({"samples": prev_samples + [_save_fields(sample[0]) for sample in samples]}, out_handle,
-                  default_flow_style=False, allow_unicode=False)
+            yaml.safe_dump({"test_run": True}, out_handle, default_flow_style=False,
+                           allow_unicode=False)
+        yaml.safe_dump({"upload": upload_dir}, out_handle,
+                       default_flow_style=False, allow_unicode=False)
+        yaml.safe_dump({"bcbio_system": samples[0][0]["config"].get("bcbio_system", "")}, out_handle,
+                       default_flow_style=False, allow_unicode=False)
+        yaml.safe_dump({"samples": prev_samples + [_save_fields(sample[0]) for sample in samples]}, out_handle,
+                       default_flow_style=False, allow_unicode=False)
     return out_file
 
 def _other_pipeline_samples(summary_file, cur_samples):
@@ -146,6 +171,44 @@ def _parse_disambiguate(disambiguatestatsfilename):
                 disambig_stats = [int(x) for x in disambig_stats_tmp]
     return disambig_stats
 
+# ## Generate researcher specific summaries
+
+def _add_researcher_summary(samples, summary_yaml):
+    """Generate summary files per researcher if organized via a LIMS.
+    """
+    by_researcher = collections.defaultdict(list)
+    for data in (x[0] for x in samples):
+        researcher = utils.get_in(data, ("upload", "researcher"))
+        if researcher:
+            by_researcher[researcher].append(data["description"])
+    out_by_researcher = {}
+    for researcher, descrs in by_researcher.items():
+        out_by_researcher[researcher] = _summary_csv_by_researcher(summary_yaml, researcher,
+                                                                   set(descrs), samples[0][0])
+    out = []
+    for data in (x[0] for x in samples):
+        researcher = utils.get_in(data, ("upload", "researcher"))
+        if researcher:
+            data["summary"]["researcher"] = out_by_researcher[researcher]
+        out.append([data])
+    return out
+
+def _summary_csv_by_researcher(summary_yaml, researcher, descrs, data):
+    """Generate a CSV file with summary information for a researcher on this project.
+    """
+    out_file = os.path.join(utils.safe_makedir(os.path.join(data["dirs"]["work"], "researcher")),
+                            "%s-summary.tsv" % run_info.clean_name(researcher))
+    metrics = ["Total reads", "Mapped reads", "Mapped reads pct", "Duplicates", "Duplicates pct"]
+    with open(summary_yaml) as in_handle:
+        with open(out_file, "w") as out_handle:
+            writer = csv.writer(out_handle, dialect="excel-tab")
+            writer.writerow(["Name"] + metrics)
+            for sample in yaml.safe_load(in_handle)["samples"]:
+                if sample["description"] in descrs:
+                    row = [sample["description"]] + [utils.get_in(sample, ("summary", "metrics", x), "")
+                                                     for x in metrics]
+                    writer.writerow(row)
+    return out_file
 
 # ## Run and parse read information from FastQC
 
@@ -193,16 +256,18 @@ def _run_fastqc(bam_file, data, fastqc_out):
     """Run fastqc, generating report in specified directory and parsing metrics.
 
     Downsamples to 10 million reads to avoid excessive processing times with large
-    files.
+    files, unless we're running a Standard/QC pipeline.
     """
     sentry_file = os.path.join(fastqc_out, "fastqc_report.html")
     if not os.path.exists(sentry_file):
         work_dir = os.path.dirname(fastqc_out)
         utils.safe_makedir(work_dir)
-        ds_bam = bam.downsample(bam_file, data, 1e7)
-        num_cores = data["config"]["algorithm"].get("num_cores", 1)
+        ds_bam = (bam.downsample(bam_file, data, 1e7)
+                  if data.get("analysis", "").lower() not in ["standard"]
+                  else None)
         bam_file = ds_bam if ds_bam else bam_file
-        with utils.curdir_tmpdir(work_dir) as tx_tmp_dir:
+        num_cores = data["config"]["algorithm"].get("num_cores", 1)
+        with utils.curdir_tmpdir(data, work_dir) as tx_tmp_dir:
             with utils.chdir(tx_tmp_dir):
                 cl = [config_utils.get_program("fastqc", data["config"]),
                       "-t", str(num_cores), "-o", tx_tmp_dir, "-f", "bam", bam_file]
