@@ -2,6 +2,7 @@
 """
 import os
 from contextlib import closing
+import functools
 
 try:
     import pybedtools
@@ -81,13 +82,14 @@ def write_noanalysis_reads(in_file, region_file, out_file, config):
     the BAM index to perform well on large files. The tricky part is avoiding
     command line limits. There is a nice discussion on SeqAnswers:
     http://seqanswers.com/forums/showthread.php?t=29538
+    sambamba supports intersection via an input BED file so avoids command line
+    length issues.
     """
     if not file_exists(out_file):
         with file_transaction(out_file) as tx_out_file:
             bedtools = config_utils.get_program("bedtools", config)
-            samtools = config_utils.get_program("samtools", config)
-            region_str = " ".join("%s:%s-%s" % tuple(r) for r in pybedtools.BedTool(region_file))
-            cl = ("{samtools} view -b {in_file} {region_str} | "
+            sambamba = config_utils.get_program("sambamba", config)
+            cl = ("{sambamba} view -f bam -L {region_file} {in_file} | "
                   "{bedtools} intersect -abam - -b {region_file} -f 1.0 "
                   "> {tx_out_file}")
             do.run(cl.format(**locals()), "Select unanalyzed reads")
@@ -127,7 +129,39 @@ def _subset_bed_by_region(in_file, out_file, region):
     region_bed = pybedtools.BedTool("\t".join(str(x) for x in region) + "\n", from_string=True)
     orig_bed.intersect(region_bed).filter(lambda x: len(x) > 5).merge().saveas(out_file)
 
-def subset_variant_regions(variant_regions, region, out_file):
+def get_lcr_bed(items):
+    lcr_bed = utils.get_in(items[0], ("genome_resources", "variation", "lcr"))
+    do_lcr = any([utils.get_in(data, ("config", "algorithm", "remove_lcr"), False)
+                  for data in items])
+    if do_lcr and lcr_bed and os.path.exists(lcr_bed):
+        return lcr_bed
+
+def remove_lcr_regions(orig_bed, items):
+    """If configured and available, update a BED file to remove low complexity regions.
+    """
+    lcr_bed = get_lcr_bed(items)
+    if lcr_bed:
+        nolcr_bed = os.path.join("%s-nolcr.bed" % (utils.splitext_plus(orig_bed)[0]))
+        with file_transaction(nolcr_bed) as tx_nolcr_bed:
+            pybedtools.BedTool(orig_bed).subtract(pybedtools.BedTool(lcr_bed)).saveas(tx_nolcr_bed)
+        # If we have a non-empty file, convert to the LCR subtracted for downstream analysis
+        if utils.file_exists(nolcr_bed):
+            orig_bed = nolcr_bed
+    return orig_bed
+
+def subtract_low_complexity(f):
+    """Remove low complexity regions from callable regions if available.
+    """
+    @functools.wraps(f)
+    def wrapper(variant_regions, region, out_file, items=None):
+        region_bed = f(variant_regions, region, out_file, items)
+        if region_bed and isinstance(region_bed, basestring) and os.path.exists(region_bed) and items:
+            region_bed = remove_lcr_regions(region_bed, items)
+        return region_bed
+    return wrapper
+
+@subtract_low_complexity
+def subset_variant_regions(variant_regions, region, out_file, items=None):
     """Return BED file subset by a specified chromosome region.
 
     variant_regions is a BED file, region is a chromosome name or tuple
