@@ -11,12 +11,13 @@ def postprocess_variants(data):
     """
     cur_name = "%s, %s" % (data["name"][-1], get_variantcaller(data))
     logger.info("Finalizing variant calls: %s" % cur_name)
-    if data["work_bam"] and data.get("vrn_file"):
-        data["vrn_file"] = variant_filtration(data["vrn_file"], data["sam_ref"],
-                                              data["genome_resources"]["variation"],
-                                              data)
+    if data.get("align_bam") and data.get("vrn_file"):
         logger.info("Calculating variation effects for %s" % cur_name)
         ann_vrn_file = effects.snpeff_effects(data)
         if ann_vrn_file:
             data["vrn_file"] = ann_vrn_file
+        logger.info("Filtering for %s" % cur_name)
+        data["vrn_file"] = variant_filtration(data["vrn_file"], data["sam_ref"],
+                                              data["genome_resources"]["variation"],
+                                              data)
     return [[data]]
