@@ -6,6 +6,8 @@ import os
 from random import shuffle
 from itertools import ifilter
 import shutil
+import uuid
+import tempfile
 
 # Provide transition period to install via upgrade with conda
 try:
@@ -34,8 +36,7 @@ class RNASeQCRunner(object):
         self._rnaseqc_path = rnaseqc_path
         self._base_cmd = ("java -jar {jvm_opts} {rnaseqc_path} -n 1000 -s "
                           "{sample_file} -t {gtf_file} "
-                          "-r {ref_file} -o {out_dir} -BWArRNA {rna_file} "
-                          "-bwa {bwa_path} -ttype 2")
+                          "-r {ref_file} -o {out_dir} -ttype 2 ")
 
     def run(self, sample_file, ref_file, rna_file, gtf_file, out_dir,
             single_end=False):
@@ -70,10 +71,10 @@ def sample_summary(bam_file, data, out_dir):
             ref_file = data["sam_ref"]
             genome_dir = os.path.dirname(os.path.dirname(ref_file))
             gtf_file = config_utils.get_transcript_gtf(genome_dir)
-            rna_file = config_utils.get_rRNA_sequence(genome_dir)
             sample_file = os.path.join(safe_makedir(tx_out_dir), "sample_file.txt")
             _write_sample_id_file(data, bam_file, sample_file)
             runner = rnaseqc_runner_from_config(config)
+            rna_file = config_utils.get_rRNA_sequence(genome_dir)
             bam.index(bam_file, config)
             single_end = not bam.is_paired(bam_file)
             runner.run(sample_file, ref_file, rna_file, gtf_file, tx_out_dir, single_end)
@@ -81,7 +82,6 @@ def sample_summary(bam_file, data, out_dir):
             shutil.rmtree(os.path.join(tx_out_dir, data["description"]))
     return _parse_rnaseqc_metrics(metrics_file, data["name"][-1])
 
-
 def _write_sample_id_file(data, bam_file, out_file):
     HEADER = "\t".join(["Sample ID", "Bam File", "Notes"]) + "\n"
     sample_ids = ["\t".join([data["description"], bam_file, data["description"]])]
