@@ -8,36 +8,41 @@ import shutil
 import subprocess
 
 import pysam
+import toolz as tz
 
 from bcbio import bam, install, utils
+from bcbio.distributed.multi import run_multicore, zeromq_aware_logging
 from bcbio.distributed.transaction import file_transaction
 from bcbio.log import logger
 from bcbio.pipeline import config_utils
 from bcbio.provenance import do
 from bcbio.variation import vcfutils
 
-def run(items):
+def run(items, background=None):
     """Detect copy number variations from batched set of samples using cn.mops.
     """
-    names = [x["name"][-1] for x in items]
-    work_bams = [x["work_bam"] for x in items]
-    if len(items) == 1:
+    if not background: background = []
+    names = [tz.get_in(["rgnames", "sample"], x) for x in items + background]
+    work_bams = [x["align_bam"] for x in items + background]
+    if len(items + background) < 2:
         raise ValueError("cn.mops only works on batches with multiple samples")
     data = items[0]
-    # XXX Can parallelize cn.mops with snow but currently causing
-    # errors. Could also parallelize by chromosome.
-    #num_cores = data["config"]["algorithm"].get("num_cores", 1)
     work_dir = utils.safe_makedir(os.path.join(data["dirs"]["work"], "structural", names[0],
                                                "cn_mops"))
+    parallel = {"type": "local", "cores": data["config"]["algorithm"].get("num_cores", 1),
+                "progs": ["delly"]}
     with closing(pysam.Samfile(work_bams[0], "rb")) as pysam_work_bam:
-        out_files = [_run_on_chrom(chrom, work_bams, names, work_dir, items)
-                     for chrom in pysam_work_bam.references]
+        chroms = [None] if _get_regional_bed_file(items[0]) else pysam_work_bam.references
+        out_files = run_multicore(_run_on_chrom, [(chrom, work_bams, names, work_dir, items)
+                                                  for chrom in chroms],
+                                  data["config"], parallel)
     out_file = _combine_out_files(out_files, work_bams[0], work_dir)
     out = []
     for data in items:
         if "sv" not in data:
-            data["sv"] = {}
-        data["sv"]["cnv"] = _prep_sample_cnvs(out_file, data)
+            data["sv"] = []
+        data["sv"].append({"variantcaller": "cn_mops",
+                           "vrn_file": _prep_sample_cnvs(out_file, data)})
         out.append(data)
     return out
 
@@ -58,22 +63,32 @@ def _combine_out_files(chr_files, base_bam, work_dir):
 
 def _prep_sample_cnvs(cnv_file, data):
     """Convert a multiple sample CNV file into a single BED file for a sample.
+
+    Handles matching and fixing names where R converts numerical IDs (1234) into
+    strings by adding an X (X1234).
     """
     import pybedtools
-    sample_name = data["name"][-1]
+    sample_name = tz.get_in(["rgnames", "sample"], data)
+    def matches_sample_name(feat):
+        return feat.name == sample_name or feat.name == "X%s" % sample_name
+    def update_sample_name(feat):
+        feat.name = sample_name
+        return feat
     sample_file = os.path.join(os.path.dirname(cnv_file), "%s-cnv.bed" % sample_name)
     if not utils.file_exists(sample_file):
         with file_transaction(sample_file) as tx_out_file:
-            pybedtools.BedTool(cnv_file).filter(lambda x: x.name == sample_name).saveas(tx_out_file)
+            pybedtools.BedTool(cnv_file).filter(matches_sample_name).each(update_sample_name).saveas(tx_out_file)
     return sample_file
 
+@utils.map_wrap
+@zeromq_aware_logging
 def _run_on_chrom(chrom, work_bams, names, work_dir, items):
     """Run cn.mops on work BAMs for a specific chromosome.
     """
     local_sitelib = os.path.join(install.get_defaults().get("tooldir", "/usr/local"),
                                  "lib", "R", "site-library")
     out_file = os.path.join(work_dir, "%s-%s-cnv.bed" % (os.path.splitext(os.path.basename(work_bams[0]))[0],
-                                                         chrom))
+                                                         chrom if chrom else "all"))
     if not utils.file_exists(out_file):
         with file_transaction(out_file) as tx_out_file:
             rcode = "%s-run.R" % os.path.splitext(out_file)[0]
@@ -92,34 +107,54 @@ def _run_on_chrom(chrom, work_bams, names, work_dir, items):
                 else:
                     logger.exception()
                     raise
-    return out_file
+    return [out_file]
 
 def _allowed_cnmops_errorstates(msg):
     return (msg.find("No CNV regions in result object. Rerun cn.mops with different parameters") >= 0
-            or msg.find("Normalization might not be applicable for this small number of segments") >= 0)
+            or msg.find("Normalization might not be applicable for this small number of segments") >= 0
+            or msg.find("Error in if (is.finite(mv2m)) { : argument is of length zero") >= 0)
 
 def _prep_load_script(work_bams, names, chrom, items):
+    if not chrom: chrom = ""
     pairmode = "paired" if bam.is_paired(work_bams[0]) else "unpaired"
-    print len(items), items[0].get("metadata")
     if len(items) == 2 and vcfutils.get_paired_phenotype(items[0]):
         load_script = _paired_load_script
     else:
         load_script = _population_load_script
     return load_script(work_bams, names, chrom, pairmode, items)
 
+def _get_regional_bed_file(data):
+    """If we are running a non-genome analysis, pull the regional file for analysis.
+    """
+    bed_file = data["config"]["algorithm"].get("variant_regions", None)
+    is_genome = data["config"]["algorithm"].get("coverage_interval", "exome").lower() in ["genome"]
+    if bed_file and utils.file_exists(bed_file) and not is_genome:
+        return bed_file
+
 def _population_load_script(work_bams, names, chrom, pairmode, items):
     """Prepare BAMs for assessing CNVs in a population.
     """
-    return _population_prep.format(bam_file_str=",".join(work_bams), names_str=",".join(names),
-                                   chrom=chrom, num_cores=0, pairmode=pairmode)
+    bed_file = _get_regional_bed_file(items[0])
+    if bed_file:
+        return _population_prep_targeted.format(bam_file_str=",".join(work_bams), names_str=",".join(names),
+                                                chrom=chrom, num_cores=0, pairmode=pairmode, bed_file=bed_file)
+    else:
+        return _population_prep.format(bam_file_str=",".join(work_bams), names_str=",".join(names),
+                                       chrom=chrom, num_cores=0, pairmode=pairmode)
 
 def _paired_load_script(work_bams, names, chrom, pairmode, items):
     """Prepare BAMs for assessing CNVs in a paired tumor/normal setup.
     """
     paired = vcfutils.get_paired_bams(work_bams, items)
-    return _paired_prep.format(case_file=paired.tumor_bam, case_name=paired.tumor_name,
-                               ctrl_file=paired.normal_bam, ctrl_name=paired.normal_name,
-                               num_cores=0, chrom=chrom, pairmode=pairmode)
+    bed_file = _get_regional_bed_file(items[0])
+    if bed_file:
+        return _paired_prep_targeted.format(case_file=paired.tumor_bam, case_name=paired.tumor_name,
+                                            ctrl_file=paired.normal_bam, ctrl_name=paired.normal_name,
+                                            num_cores=0, chrom=chrom, pairmode=pairmode, bed_file=bed_file)
+    else:
+        return _paired_prep.format(case_file=paired.tumor_bam, case_name=paired.tumor_name,
+                                   ctrl_file=paired.normal_bam, ctrl_name=paired.normal_name,
+                                   num_cores=0, chrom=chrom, pairmode=pairmode)
 
 _script = """
 .libPaths(c("{local_sitelib}"))
@@ -154,3 +189,29 @@ ctrl_count <- getReadCountsFromBAM(c("{ctrl_file}"), sampleNames=c("{ctrl_name}"
 prep_counts <- referencecn.mops(case_count, ctrl_count, parallel={num_cores})
 cnv_out <- calcFractionalCopyNumbers(prep_counts)
 """
+
+_population_prep_targeted = """
+bam_files <- strsplit("{bam_file_str}", ",")[[1]]
+sample_names <- strsplit("{names_str}", ",")[[1]]
+my_gr <- import.bed(c("{bed_file}"), trackLine=FALSE, asRangedData=FALSE)
+if ("{chrom}" != "") my_gr = subset(my_gr, seqnames(my_gr) == "{chrom}")
+if (length(my_gr) < 1) stop("No CNV regions in result object. Rerun cn.mops with different parameters!")
+count_drs <- getSegmentReadCountsFromBAM(bam_files, sampleNames=sample_names, mode="{pairmode}",
+                                         GR=my_gr, parallel={num_cores})
+prep_counts <- cn.mops(count_drs, parallel={num_cores})
+cnv_out <- calcIntegerCopyNumbers(prep_counts)
+"""
+
+_paired_prep_targeted = """
+my_gr <- import.bed(c("{bed_file}"), trackLine=FALSE, asRangedData=FALSE)
+if ("{chrom}" != "") my_gr = subset(my_gr, seqnames(my_gr) == "{chrom}")
+if (length(my_gr) < 1) stop("No CNV regions in result object. Rerun cn.mops with different parameters!")
+case_count <- getSegmentReadCountsFromBAM(c("{case_file}"), GR=my_gr,
+                                          sampleNames=c("{case_name}"),
+                                          mode="{pairmode}", parallel={num_cores})
+ctrl_count <- getSegmentReadCountsFromBAM(c("{ctrl_file}"), GR=my_gr,
+                                          sampleNames=c("{case_name}"),
+                                          mode="{pairmode}", parallel={num_cores})
+prep_counts <- referencecn.mops(case_count, ctrl_count, parallel={num_cores})
+cnv_out <- calcFractionalCopyNumbers(prep_counts)
+"""
