@@ -3,8 +3,9 @@
 import datetime
 import os
 
-from bcbio import utils
+from bcbio import log, utils
 from bcbio.upload import shared, filesystem, galaxy, s3
+from bcbio.pipeline import run_info
 
 _approaches = {"filesystem": filesystem,
                "galaxy": galaxy,
@@ -30,7 +31,7 @@ def _get_files(sample):
     metadata about the file and pipeline versions.
     """
     analysis = sample.get("analysis")
-    if analysis in ["variant", "SNP calling", "variant2"]:
+    if analysis.lower() in ["variant", "snp calling", "variant2", "standard"]:
         return _get_files_variantcall(sample)
     elif analysis in ["RNA-seq"]:
         return _get_files_rnaseq(sample)
@@ -46,6 +47,7 @@ def _get_files_rnaseq(sample):
     out = _maybe_add_alignment(algorithm, sample, out)
     out = _maybe_add_counts(algorithm, sample, out)
     out = _maybe_add_cufflinks(algorithm, sample, out)
+    out = _maybe_add_oncofuse(algorithm, sample, out)
     return _add_meta(out, sample)
 
 def _get_files_chipseq(sample):
@@ -59,8 +61,12 @@ def _add_meta(xs, sample=None, config=None):
     out = []
     for x in xs:
         x["mtime"] = shared.get_file_timestamp(x["path"])
-        if sample:
-            x["sample"] = sample["name"][-1]
+        if sample and "sample" not in x:
+            if isinstance(sample["name"], (tuple, list)):
+                name = sample["name"][-1]
+            else:
+                name = "%s-%s" % (sample["name"], run_info.clean_name(sample["description"]))
+            x["sample"] = name
         if config:
             if "fc_name" in config and "fc_date" in config:
                 x["run"] = "%s_%s" % (config["fc_date"], config["fc_name"])
@@ -77,12 +83,13 @@ def _get_files_variantcall(sample):
     out = _maybe_add_summary(algorithm, sample, out)
     out = _maybe_add_alignment(algorithm, sample, out)
     out = _maybe_add_variant_file(algorithm, sample, out)
+    out = _maybe_add_sv(algorithm, sample, out)
     return _add_meta(out, sample)
 
 def _maybe_add_variant_file(algorithm, sample, out):
-    if sample["work_bam"] is not None and sample.get("vrn_file"):
+    if sample.get("align_bam") is not None and sample.get("vrn_file"):
         for x in sample["variants"]:
-            out.extend(_get_vcf(x, ("vrn_file",)))
+            out.extend(_get_variant_file(x, ("vrn_file",)))
             if x.get("bed_file"):
                 out.append({"path": x["bed_file"],
                             "type": "bed",
@@ -90,13 +97,19 @@ def _maybe_add_variant_file(algorithm, sample, out):
                             "variantcaller": x["variantcaller"]})
     return out
 
-def _get_vcf(x, key):
+def _maybe_add_sv(algorithm, sample, out):
+    if sample.get("align_bam") is not None and sample.get("sv"):
+        for svcall in sample["sv"]:
+            out.extend(_get_variant_file(svcall, ("vrn_file",)))
+    return out
+
+def _get_variant_file(x, key):
     """Retrieve VCF file with the given key if it exists, handling bgzipped.
     """
     out = []
     fname = utils.get_in(x, key)
     if fname:
-        if fname.endswith(".gz"):
+        if fname.endswith(".vcf.gz"):
             out.append({"path": fname,
                         "type": "vcf.gz",
                         "ext": x["variantcaller"],
@@ -104,11 +117,13 @@ def _get_vcf(x, key):
             if utils.file_exists(fname + ".tbi"):
                 out.append({"path": fname + ".tbi",
                             "type": "vcf.gz.tbi",
+                            "index": True,
                             "ext": x["variantcaller"],
                             "variantcaller": x["variantcaller"]})
-        else:
+        elif fname.endswith((".vcf", ".bed", ".bedpe")):
+            ftype = utils.splitext_plus(fname)[-1][1:]
             out.append({"path": fname,
-                        "type": "vcf",
+                        "type": ftype,
                         "ext": x["variantcaller"],
                         "variantcaller": x["variantcaller"]})
     return out
@@ -124,17 +139,33 @@ def _maybe_add_summary(algorithm, sample, out):
             out.append({"path": sample["summary"]["qc"],
                         "type": "directory",
                         "ext": "qc"})
+        if utils.get_in(sample, ("summary", "researcher")):
+            out.append({"path": sample["summary"]["researcher"],
+                        "type": "tsv",
+                        "sample": run_info.clean_name(utils.get_in(sample, ("upload", "researcher"))),
+                        "ext": "summary"})
     return out
 
 def _maybe_add_alignment(algorithm, sample, out):
     if _has_alignment_file(algorithm, sample):
-        out.append({"path": sample["work_bam"],
-                    "type": "bam",
-                    "ext": "ready"})
-        if utils.file_exists(sample["work_bam"] + ".bai"):
-            out.append({"path": sample["work_bam"] + ".bai",
-                        "type": "bam.bai",
-                        "ext": "ready"})
+        for (fname, ext) in [(sample.get("work_bam"), "ready"),
+                             (utils.get_in(sample, ("work_bam-plus", "disc")), "disc"),
+                             (utils.get_in(sample, ("work_bam-plus", "sr")), "sr")]:
+            if fname and os.path.exists(fname):
+                if fname.endswith("bam"):
+                    ftype, fext = "bam", ".bai"
+                elif fname.endswith("cram"):
+                    ftype, fext = "cram", ".crai"
+                else:
+                    raise ValueError("Unexpected alignment file type %s" % fname)
+                out.append({"path": fname,
+                            "type": ftype,
+                            "ext": ext})
+                if utils.file_exists(fname + fext):
+                    out.append({"path": fname + fext,
+                                "type": ftype + fext,
+                                "index": True,
+                                "ext": ext})
     return out
 
 def _maybe_add_counts(algorithm, sample, out):
@@ -148,6 +179,13 @@ def _maybe_add_counts(algorithm, sample, out):
                     "ext": "ready"})
     return out
 
+def _maybe_add_oncofuse(algorithm, sample, out):
+    if sample.get("oncofuse_file", None) is not None:
+        out.append({"path": sample["oncofuse_file"],
+                    "type": "oncofuse_outfile",
+                    "ext": "ready"})
+    return out
+
 def _maybe_add_cufflinks(algorithm, sample, out):
     if "cufflinks_dir" in sample:
         out.append({"path": sample["cufflinks_dir"],
@@ -159,7 +197,7 @@ def _has_alignment_file(algorithm, sample):
     return (((algorithm.get("aligner") or algorithm.get("realign")
               or algorithm.get("recalibrate")) and
               algorithm.get("merge_bamprep", True)) and
-              sample["work_bam"] is not None)
+              sample.get("work_bam") is not None)
 
 # ## File information from full project
 
@@ -167,6 +205,12 @@ def _get_files_project(sample, upload_config):
     """Retrieve output files associated with an entire analysis project.
     """
     out = [{"path": sample["provenance"]["programs"]}]
+    for fname in ["bcbio-nextgen.log", "bcbio-nextgen-commands.log"]:
+        if os.path.exists(os.path.join(log.get_log_dir(sample["config"]), fname)):
+            out.append({"path": os.path.join(log.get_log_dir(sample["config"]), fname),
+                        "type": "external_command_log",
+                        "ext": ""})
+
     if "summary" in sample and sample["summary"].get("project"):
         out.append({"path": sample["summary"]["project"]})
 
@@ -182,7 +226,7 @@ def _get_files_project(sample, upload_config):
                 out.append({"path": pop_db,
                             "type": "sqlite",
                             "variantcaller": x["variantcaller"]})
-            out.extend(_get_vcf(x, ("population", "vcf")))
+            out.extend(_get_variant_file(x, ("population", "vcf")))
     for x in sample.get("variants", []):
         if x.get("validate") and x["validate"].get("grading_summary"):
             out.append({"path": x["validate"]["grading_summary"]})
