@@ -160,12 +160,15 @@ def safe_makedir(dname):
     return dname
 
 @contextlib.contextmanager
-def curdir_tmpdir(remove=True, base_dir=None):
+def curdir_tmpdir(data=None, base_dir=None, remove=True):
     """Context manager to create and remove a temporary directory.
 
     This can also handle a configured temporary directory to use.
     """
-    if base_dir is not None:
+    config_tmpdir = get_in(data, ("config", "resources", "tmp", "dir")) if data else None
+    if config_tmpdir:
+        tmp_dir_base = os.path.join(config_tmpdir, "bcbiotmp")
+    elif base_dir is not None:
         tmp_dir_base = os.path.join(base_dir, "bcbiotmp")
     else:
         tmp_dir_base = os.path.join(os.getcwd(), "tmp")
@@ -210,7 +213,7 @@ def tmpfile(*args, **kwargs):
 def file_exists(fname):
     """Check if a file exists and is non-empty.
     """
-    return os.path.exists(fname) and os.path.getsize(fname) > 0
+    return fname and os.path.exists(fname) and os.path.getsize(fname) > 0
 
 def file_uptodate(fname, cmp_fname):
     """Check if a file exists, is non-empty and is more recent than cmp_fname.
@@ -376,7 +379,9 @@ def get_in(d, t, default=None):
     example: get_in({1: {2: 3}}, (2, 3)) -> {}
     """
     result = reduce(lambda d, t: d.get(t, {}), t, d)
-    if not result:
+    if result is False:
+        return result
+    elif not result:
         return default
     else:
         return result
