@@ -5,7 +5,6 @@ database of variations for query and evaluation.
 """
 import collections
 from distutils.version import LooseVersion
-import glob
 import os
 import subprocess
 
@@ -15,9 +14,10 @@ from bcbio.pipeline import config_utils
 from bcbio.provenance import do, programs
 from bcbio.variation import vcfutils
 
-def prep_gemini_db(fnames, call_info, samples, data):
+def prep_gemini_db(fnames, call_info, samples):
     """Prepare a gemini database from VCF inputs prepared with snpEff.
     """
+    data = samples[0]
     out_dir = utils.safe_makedir(os.path.join(data["dirs"]["work"], "gemini"))
     name, caller, is_batch = call_info
     gemini_db = os.path.join(out_dir, "%s-%s.db" % (name, caller))
@@ -75,8 +75,16 @@ def get_multisample_vcf(fnames, name, caller, data):
     out_dir = utils.safe_makedir(os.path.join(data["dirs"]["work"], "gemini"))
     if len(unique_fnames) > 1:
         gemini_vcf = os.path.join(out_dir, "%s-%s.vcf.gz" % (name, caller))
-        return vcfutils.merge_variant_files(unique_fnames, gemini_vcf, data["sam_ref"],
-                                            data["config"])
+        vrn_file_batch = None
+        for variant in data["variants"]:
+            if variant["variantcaller"] == caller and variant.get("vrn_file_batch"):
+                vrn_file_batch = variant["vrn_file_batch"]
+        if vrn_file_batch:
+            utils.symlink_plus(vrn_file_batch, gemini_vcf)
+            return gemini_vcf
+        else:
+            return vcfutils.merge_variant_files(unique_fnames, gemini_vcf, data["sam_ref"],
+                                                data["config"])
     else:
         gemini_vcf = os.path.join(out_dir, "%s-%s%s" % (name, caller, utils.splitext_plus(unique_fnames[0])[1]))
         utils.symlink_plus(unique_fnames[0], gemini_vcf)
@@ -98,12 +106,14 @@ def _has_gemini(config):
     return True
 
 def do_db_build(samples, check_gemini=True, need_bam=True):
-    """Confirm we should build a gemini database: need gemini + human samples.
+    """Confirm we should build a gemini database: need gemini + human samples + not in tool_skip.
     """
     genomes = set()
     for data in samples:
-        if not need_bam or data.get("work_bam"):
+        if not need_bam or data.get("align_bam"):
             genomes.add(data["genome_build"])
+        if "gemini" in utils.get_in(data, ("config", "algorithm", "tools_off"), []):
+            return False
     if len(genomes) == 1:
         return (samples[0]["genome_resources"].get("aliases", {}).get("human", False)
                 and (not check_gemini or _has_gemini(samples[0]["config"])))
@@ -145,7 +155,7 @@ def _group_by_batches(samples, check_fn):
     return batch_groups, singles, out_retrieve, extras
 
 def _has_variant_calls(data):
-    return data["work_bam"] and data.get("vrn_file") and vcfutils.vcf_has_variants(data["vrn_file"])
+    return data.get("align_bam") and data.get("vrn_file") and vcfutils.vcf_has_variants(data["vrn_file"])
 
 def prep_db_parallel(samples, parallel_fn):
     """Prepares gemini databases in parallel, handling jointly called populations.
@@ -155,10 +165,10 @@ def prep_db_parallel(samples, parallel_fn):
     has_batches = False
     for (name, caller), info in batch_groups.iteritems():
         fnames = [x[0] for x in info]
-        to_process.append([fnames, (str(name), caller, True), [x[1] for x in info], info[0][1]])
+        to_process.append([fnames, (str(name), caller, True), [x[1] for x in info]])
         has_batches = True
     for name, caller, data, fname in singles:
-        to_process.append([[fname], (str(name), caller, False), [data], data])
+        to_process.append([[fname], (str(name), caller, False), [data]])
     if len(samples) > 0 and not do_db_build([x[0] for x in samples], check_gemini=False) and not has_batches:
         return samples
     output = parallel_fn("prep_gemini_db", to_process)
