@@ -85,9 +85,9 @@ class AutomatedAnalysisTest(unittest.TestCase):
         """
         DlInfo = collections.namedtuple("DlInfo", "fname dirname version")
         download_data = [DlInfo("110106_FC70BUKAAXX.tar.gz", None, None),
-                         DlInfo("genomes_automated_test.tar.gz", "genomes", 14),
+                         DlInfo("genomes_automated_test.tar.gz", "genomes", 17),
                          DlInfo("110907_ERP000591.tar.gz", None, None),
-                         DlInfo("100326_FC6107FAAXX.tar.gz", None, 5),
+                         DlInfo("100326_FC6107FAAXX.tar.gz", None, 8),
                          DlInfo("tcga_benchmark.tar.gz", None, 2)]
         for dl in download_data:
             url = "http://chapmanb.s3.amazonaws.com/{fname}".format(fname=dl.fname)
@@ -168,7 +168,6 @@ class AutomatedAnalysisTest(unittest.TestCase):
                   os.path.join(self.data_dir, "run_info-rnaseq.yaml")]
             subprocess.check_call(cl)
 
-    @expected_failure
     @attr(fusion=True)
     def test_2_fusion(self):
         """Run an RNA-seq analysis and test fusion genes
@@ -176,8 +175,7 @@ class AutomatedAnalysisTest(unittest.TestCase):
         self._install_test_files(self.data_dir)
         with make_workdir() as workdir:
             cl = ["bcbio_nextgen.py",
-                  "/usr/local/share/bcbio-nextgen/galaxy/bcbio_system.yaml",
-                  # get_post_process_yaml(self.data_dir, workdir),
+                  get_post_process_yaml(self.data_dir, workdir),
                   os.path.join(self.data_dir, os.pardir, "test_fusion"),
                   os.path.join(self.data_dir, "run_info-fusion.yaml")]
             subprocess.check_call(cl)
